#!/usr/bin/env node

"use strict";

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var pkg = require("../package.json");
var program = require("commander");
var Table = require("cli-table");
var rp = require("request-promise");
var fsp = require("fs-promise");

require("colors");

var main = function main() {
	Promise.resolve().then(setup).then(process_options).then(run).then(save).catch(function (err) {
		process.stdin.destroy();
		process.stdout.write("Error\n".red);

		if (err.error) {
			console.error(JSON.parse(err.error).error.message);
		} else {
			console.error(err);
		}
		return false;
	})
	// done
	.then(display_results).then(function () {
		process.stdout.write("\n");
	}).catch(function (err) {
		process.stdin.destroy();
		console.error(err);
	});
};

var setup = function setup() {
	program.version(pkg.version).arguments("<ACCESS-TOKEN>").action(function (token) {
		program.token = token;
	}).usage('<ACCESS-TOKEN> [options]').option("-l, --list", "List Facebook Groups").option("-g, --groupId [id]", "ID for group to export").option("-f, --file [path]", "File to store data in").parse(process.argv);

	program.on("--help", function () {
		console.log("");
		console.log("");
		console.log("  Examples:");
		console.log("");
		console.log("    facebook-group-export <ACCESS-TOKEN> -l");
		console.log("    facebook-group-export <ACCESS-TOKEN> -l -f ./path/file.csv");
		console.log("");
		console.log("    facebook-group-export <ACCESS-TOKEN> -g <GROUP-ID>");
		console.log("    facebook-group-export <ACCESS-TOKEN> -g <GROUP-ID> -f ./path/file.csv");
		console.log("");
	});
};

var process_options = function process_options() {
	var paramaters = {
		accessToken: null,
		groupId: null,
		file: null,
		list: false
	};

	if (program.token !== undefined) paramaters.accessToken = program.token;
	if (program.groupId !== undefined) paramaters.groupId = program.groupId;
	if (program.file !== undefined) paramaters.file = program.file;
	paramaters.list = program.list === true;

	return paramaters;
};

var run = function run(paramaters) {
	if (!(paramaters.accessToken !== null && (paramaters.list || paramaters.groupId !== null))) {
		program.help();
		throw "Missing Required Paramaters.";
	}

	process.stdout.write("\n");

	if (paramaters.list) {
		return list_groups(paramaters);
	} else if (paramaters.groupId) {
		return export_group(paramaters);
	}
};

var list_groups = function list_groups(paramaters) {
	if (!paramaters.list) {
		return;
	}

	return next_loop(paramaters, "https://graph.facebook.com/v2.8/me/groups?fields=name%2Cadministrator");
};

var export_group = function export_group(paramaters) {
	if (paramaters.groupId === null) {
		return;
	}

	return next_loop(paramaters, "https://graph.facebook.com/v2.8/" + paramaters.groupId + "/members?fields=id%2Cfirst_name%2Cmiddle_name%2Clast_name%2Cadministrator");
};

var next_loop = function next_loop(paramaters, url, rows) {
	var page = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 1;

	if (!url.includes(paramaters.accessToken)) {
		url += "&access_token=" + paramaters.accessToken;
	}

	if (rows == null) {
		rows = [];
	}

	return rp(url).then(function (data) {
		return JSON.parse(data);
	}).then(function (results) {
		var paging = results.paging;
		results.data.forEach(function (result) {
			rows.push(result);
		});

		if (paging && paging.next) {

			if (page === 1) {
				process.stdout.write("More than 25 Results, Paging is required." + "\n");
				process.stdout.write("This could take a very long time for large groups." + "\n");
				process.stdout.write("Hit " + "Control-C".red + " to quit." + "\n");
				process.stdout.write("\n");
			}

			process.stdout.clearLine();
			process.stdout.cursorTo(0);
			process.stdout.write("Pages Queried: " + page.toString().green + ", Results Found: " + rows.length.toString().green);

			return next_loop(paramaters, paging.next, rows, page + 1);
		}

		if (rows.length > 25) {
			process.stdout.write("\n" + "\n");
		}

		return [rows, paramaters];
	});
};

var save = function save() {
	var _arguments$ = _slicedToArray(arguments[0], 2),
	    rows = _arguments$[0],
	    paramaters = _arguments$[1];

	if (paramaters.file == null) {
		return rows;
	}

	var keys = Object.keys(rows[0]);

	var promises = [fsp.writeFile(paramaters.file, keys.join(",") + "\n")];

	rows.forEach(function (row) {
		promises.push(fsp.appendFile(paramaters.file, keys.map(function (key) {
			return row[key];
		}).join(",") + "\n"));
	});

	return Promise.all(promises).then(function () {
		return fsp.realpath(paramaters.file);
	}).then(function (path) {
		process.stdout.write("All Data writen to " + path.green + "\n");
		return rows;
	});
};

var display_results = function display_results(rows) {
	if (!rows) {
		return;
	}
	if (rows.length > 100) {
		if (rows[0].administrator != null) {
			process.stdout.write("\n" + "Displaying Administrators".green + "\n");
			display_results(rows.filter(function (row) {
				return row.administrator;
			}));
		}
		process.stdout.write("\n" + "Displaying First 50 rows".green + " (Export to file to see all members)\n");
		rows = rows.splice(0, 50);
	}
	var keys = Object.keys(rows[0]);
	var maxWidthByKey = {};
	keys.forEach(function (key) {
		maxWidthByKey[key] = key.length;
		rows.forEach(function (row) {
			if (row[key].length > maxWidthByKey[key]) {
				maxWidthByKey[key] = row[key].length;
			}
		});
	});

	var options = {

		head: keys,
		colWidths: keys.map(function (key) {
			return maxWidthByKey[key] + 2;
		}),
		chars: { "mid": "", "left-mid": "", "mid-mid": "", "right-mid": "" },
		style: { head: ["white"], border: ["white"] }

	};

	var table = new Table(options);
	rows.forEach(function (row) {
		table.push(keys.map(function (key) {
			if (typeof row[key] === "boolean") {
				row[key] = row[key] ? "True" : "False";
			}
			return row[key];
		}));
	});

	// Print table.
	process.stdout.write(table.toString() + "\n");
};

main();